@@ -124,13 +124,13 @@ class SimpleTokenizer(object):
             bpe_tokens.extend(self.encoder[bpe_token] for bpe_token in self.bpe(token).split(' '))
         return bpe_tokens
 
-    def decode(self, tokens, remove_start_end = True):
+    def decode(self, tokens, remove_start_end = True, pad_tokens = {}):
         if torch.is_tensor(tokens):
             tokens = tokens.tolist()
 
         if remove_start_end:
             tokens = [token for token in tokens if token not in (49406, 40407, 0)]
-        text = ''.join([self.decoder[token] for token in tokens])
+        text = ''.join([self.decoder[token] for token in tokens if token not in pad_tokens])
         text = bytearray([self.byte_decoder[c] for c in text]).decode('utf-8', errors="replace").replace('</w>', ' ')
         return text
 
@@ -164,11 +164,11 @@ class HugTokenizer:
         self.tokenizer = tokenizer
         self.vocab_size = tokenizer.get_vocab_size()
 
-    def decode(self, tokens):
+    def decode(self, tokens, pad_tokens = {}):
         if torch.is_tensor(tokens):
             tokens = tokens.tolist()
-
-        tokens = [token for token in tokens if token not in (0,)]
+        ignore_ids = pad_tokens.union({0})
+        tokens = [token for token in tokens if token not in ignore_ids]
         return self.tokenizer.decode(tokens, skip_special_tokens = True)
 
     def encode(self, text):
@@ -199,11 +199,12 @@ class ChineseTokenizer:
         self.tokenizer = tokenizer
         self.vocab_size = tokenizer.vocab_size
 
-    def decode(self, tokens):
+    def decode(self, tokens, pad_tokens = {}):
         if torch.is_tensor(tokens):
             tokens = tokens.tolist()
-
-        tokens = [token for token in tokens if token not in (0,)]
+            
+        ignore_ids = pad_tokens.union({0})
+        tokens = [token for token in tokens if token not in ignore_ids]
         return self.tokenizer.decode(tokens)
 
     def encode(self, text):
@@ -237,11 +238,11 @@ class YttmTokenizer:
         self.tokenizer = tokenizer
         self.vocab_size = tokenizer.vocab_size()
 
-    def decode(self, tokens):
+    def decode(self, tokens, pad_tokens = {}):
         if torch.is_tensor(tokens):
             tokens = tokens.tolist()
 
-        return self.tokenizer.decode(tokens, ignore_ids = [0])
+        return self.tokenizer.decode(tokens, ignore_ids = pad_tokens.union({0}))
 
     def encode(self, texts):
         encoded = self.tokenizer.encode(texts, output_type = yttm.OutputType.ID)
